@@ -10,7 +10,7 @@ const int be_default_protocol = PROT_TELNET;
 
 const char *const appname = "PuTTYtel";
 
-Backend *backends[] = {
+const struct BackendVtable *const backends[] = {
     &telnet_backend,
     &rlogin_backend,
     &raw_backend,
@@ -29,6 +29,6 @@ void random_destroy_seed(void)
 {
 }
 
-void noise_ultralight(unsigned long data)
+void noise_ultralight(NoiseSourceId id, unsigned long data)
 {
 }
