@@ -13,14 +13,14 @@ text=`{ find . -name CVS -prune -o \
                -name '*.dsp' -prune -o \
                -name '*.dsw' -prune -o \
                -type f -print | sed 's/^\.\///'; } | \
-      grep -ivE 'testdata/.*\.txt|MODULE|putty.iss|website.url' | grep -vF .ico | grep -vF .icns`
+      grep -ivE 'test/.*\.txt|MODULE|website.url' | grep -vF .ico | grep -vF .icns`
 # These are files which I'm _sure_ should be treated as text, but
 # which zip might complain about, so we direct its moans to
 # /dev/null! Apparently its heuristics are doubtful of UTF-8 text
 # files.
-bintext=testdata/*.txt
+bintext=test/*.txt
 # These are actual binary files which we don't want transforming.
-bin=`{ ls -1 windows/*.ico windows/putty.iss windows/website.url; \
+bin=`{ ls -1 windows/*.ico windows/website.url; \
        find . -name '*.dsp' -print -o -name '*.dsw' -print; }`
 
 verbosely() {
